@@ -1,5 +1,9 @@
 from lqreports.constants import LinkType
-
+from lqreports.util import dataurl, mimetype_from_extension
+from lqreports.resource import resources_path
+import pandas as pd
+import numpy as np
+from io import BytesIO
 
 class RenderContext(object):
     def __init__(self, link_type=LinkType.LINK):
@@ -147,6 +151,141 @@ class VuetifyApp(Segment):
 """
 
 
+class VuetifyPanel(Segment):
+    def __init__(self, name, register, fluid=False):
+        super().__init__(
+            name=name,
+            register=register,
+            prefix=f"""<v-container {"fluid" if fluid else ""} v-if='visible_panel=="{name}"'>\n""",
+            suffix="""</v-container>\n""",
+        )
+
+    def dataframe_view(
+        self,
+        name="dataframe",
+        items_per_page=10,
+        attr='class="elevation-1"',
+        show_select=False,
+        single_select=True,
+        row_action_icon="mdi-eye"
+    ):
+        code = ""
+        if show_select:
+            code += f""" show-select single-select='{"true" if single_select else "false"}' """
+            code += """ v-model='selected' """
+        template_code = """
+        <template v-slot:item.0="{ item }">
+            <v-btn @click="row_action(item[0])" icon>
+                <v-icon>%s</v-icon>
+            </v-btn>
+        </template>
+        """%row_action_icon
+        self.add(
+            f"""
+        <v-card>
+            <v-card-title>
+            <v-text-field
+                v-model="search"
+                append-icon="mdi-magnify"
+                label="Search"
+                single-line
+                hide-details
+            ></v-text-field>
+            </v-card-title>
+            <v-data-table
+                %s
+                :headers="{name}_headers"
+                :items="{name}_data"
+                :items-per-page="{items_per_page}"
+                {attr}
+                :search="search"
+            >
+            {template_code}
+            </v-data-table>
+        </v-card>        
+        """
+            % code
+        )
+
+    def row_detail(self, title_index=1):
+        import html
+        #df = self.register.document.df
+        labels = self.register.document.labels
+        html = """
+        <v-card>
+          <v-card-title>{{selected_row[%d]}}</v-card-title>
+          <v-card-text>
+%s
+          </v-card-text>
+        </v-card>
+        """%(title_index, "\n".join(f"""
+            <v-row>
+              <v-col>{html.escape(label)}:</v-col>
+              <v-col>%s</v-col>
+            </v-row>
+        """%("{{selected_row[%d]}}"%i) for i, label in enumerate(labels) if i>0))
+        self.add(html)
+        return self
+
+    def switch(self, model, label=None, value=None):
+        if label is None:
+            label = model
+        if value is not None:
+            self.register.vuetify_script.add_data(model, value)
+        self.add(f"<v-switch v-model='{model}' label='{label}'></v-switch>")
+        return self
+
+    def checkbox(self, model, label=None, value=None):
+        if label is None:
+            label = model
+        if value is not None:
+            self.register.vuetify_script.add_data(model, value)
+        self.add(f"<v-checkbox v-model='{model}' label='{label}'></v-checkbox>")
+        return self
+
+    def textfield(self, model, label=None, value=None, hint=None, attr=""):
+        if label is None:
+            label = model
+        if value is not None:
+            self.register.vuetify_script.add_data(model, value)
+        if hint is not None:
+            attr+=f" hint='{hint}'"
+        self.add(f"<v-text-field v-model='{model}' label='{label}' {attr}></v-text-field>")
+        return self
+
+    def textfield(self, model, label=None, value=None, hint=None, attr=""):
+        if label is None:
+            label = model
+        if value is not None:
+            self.register.vuetify_script.add_data(model, value)
+        if hint is not None:
+            attr+=f" hint='{hint}'"
+        self.add(f"<v-textarea v-model='{model}' label='{label}' {attr}></v-textarea>")
+        return self
+
+    
+    def figure(self, fig, extension="svg", max_width=800, max_height=600):
+        assert extension in ("png", "svg", "pdf", "ps", "eps")
+        output = BytesIO()
+        fig.savefig(output, dpi=300, format=extension)
+        url = dataurl(output.getvalue(), mimetype_from_extension(extension))
+        html = f"""<v-img src="{url}" max-height="{max_height}" max-width="{max_width}"></v-img>"""
+        self.add(html)
+        return self
+
+    def image(self, path, max_width=800, max_height=600):
+        extension = path.split(".")[-1]
+        assert extension in ("png", "svg", "jpg", "jpeg", "gif")
+        data = open(path,"rb").read()
+        url = dataurl(data, mimetype_from_extension(extension))
+        html = f"""<v-img src="{url}" max-height="{max_height}" max-width="{max_width}"></v-img>"""
+        self.add(html)
+        return self
+    
+    def liquer_logo(self):
+        path = str(resources_path() / "liquer.png")
+        return self.image(path, max_width=500, max_height=707)
+
 class VuetifyScript(Segment):
     def __init__(self, register):
         super().__init__("vuetify_script", register)
@@ -165,8 +304,7 @@ class VuetifyScript(Segment):
       data: {
 """
         )
-        vue_data = Segment("vue_data", r)
-        vue_data.separator = ",\n"
+        vue_data = Segment("vue_data", r, separator=",\n")
         self.add(vue_data)
         self.add(
             """
@@ -174,14 +312,14 @@ class VuetifyScript(Segment):
       methods: {
 """
         )
-        self.add(Segment("vue_methods", r))
+        self.add(Segment("vue_methods", r, separator=",\n"))
         self.add(
             """
       },
       computed: {
 """
         )
-        self.add(Segment("vue_computed", r))
+        self.add(Segment("vue_computed", r, separator=",\n"))
         self.add(
             """
       },
@@ -191,9 +329,16 @@ class VuetifyScript(Segment):
         self.add(Segment("vue_created", r))
         self.add(
             """
-      }
+      },
+      watch: {
 """
         )
+        self.add(Segment("vue_watch", r, separator=",\n"))
+        self.add(
+            """
+      }
+""")
+
         self.add(Segment("vue_other", r))
         self.add(
             """
@@ -243,6 +388,10 @@ class VuetifyScript(Segment):
         self.register.vue_computed.add(f"        {name}: {code}")
         return self
 
+    def add_watch(self, name, function):
+        self.register.vue_watch.add(f"        {name}: {function}")
+        return self
+
     def add_created(self, code):
         self.register.vue_created.add(code)
         return self
@@ -254,6 +403,9 @@ class VuetifyDocument(HtmlDocument):
         self.register.header.add_resource("materialdesignicons")
         self.register.header.add_resource("vuetify_css")
         self.register.content.add(VuetifyApp("app", register))
+        self.register.app.add(
+            Segment("v_main", register, prefix="<v-main>\n", suffix="</v-main>\n")
+        )
         self.register.scripts.add_resource("vue")
         self.register.scripts.add_resource("vue_resource")
         self.register.scripts.add_resource("vuetify")
@@ -280,6 +432,23 @@ class VuetifyDashboard(VuetifyDocument):
         )
         return self
 
+    def with_panels(self):
+        r = self.register
+        r.app.add(Segment("panels", r))
+        r.vuetify_script.add_data("visible_panel", "home_panel")
+        r.vuetify_script.add_method(
+            "show_panel",
+            """function(panel){console.log("Show",panel);this.visible_panel=panel;}""",
+        )
+        self.panel("home_panel")
+        return self
+
+    def panel(self, name, fluid=False):
+        r = self.register
+        panel = VuetifyPanel(name, r, fluid=fluid)
+        r.v_main.add(panel)
+        return panel
+
     def add_drawer_item_raw(self, entry):
         if "navigation_drawer" not in self.register:
             self.with_navigation_drawer()
@@ -293,9 +462,14 @@ class VuetifyDashboard(VuetifyDocument):
         color = arg.get("color")
         style = arg.get("style")
         attr = arg.get("attr")
+        panel = arg.get("panel")
 
         item_attributes = ""
         if click is not None:
+            if panel is not None:
+                raise Exception(
+                    f"Using both click and panel at the same time is not supported."
+                )
             item_attributes += f""" @click="{click}" """
         if href is not None:
             item_attributes += f""" href="{href}" """
@@ -307,6 +481,13 @@ class VuetifyDashboard(VuetifyDocument):
             item_attributes += f""" style="{style}" """
         if attr is not None:
             item_attributes += f""" {attr} """
+        if panel is not None:
+            if "panels" not in self.register:
+                raise Exception(f"Panels not available, use .with_panels() method.")
+            if panel not in self.register:
+                raise Exception(f"Panel {panel} not available, use .panel() method.")
+            item_attributes += f""" @click="show_panel('{panel}')" """
+
         return item_attributes
 
     def drawer_item(
@@ -319,6 +500,7 @@ class VuetifyDashboard(VuetifyDocument):
         color=None,
         style=None,
         attr=None,
+        panel=None,
     ):
         if "navigation_drawer" not in self.register:
             self.with_navigation_drawer()
@@ -331,6 +513,7 @@ class VuetifyDashboard(VuetifyDocument):
                 color=color,
                 style=style,
                 attr=attr,
+                panel=panel,
             )
         )
 
@@ -341,8 +524,8 @@ class VuetifyDashboard(VuetifyDocument):
         )
         text = f"""
         <v-list-item {item_attributes}>
-        {icon_code}
-        <v-list-item-title>{title}</v-list-item-title>
+            {icon_code}
+            <v-list-item-title>{title}</v-list-item-title>
         </v-list-item>
         """
         self.register.navigation_drawer.add(text)
@@ -382,6 +565,7 @@ class VuetifyDashboard(VuetifyDocument):
         color=None,
         style=None,
         attr=None,
+        panel=None,
     ):
         if "app_bar" not in self.register:
             self.with_app_bar()
@@ -394,6 +578,7 @@ class VuetifyDashboard(VuetifyDocument):
                 color=color,
                 style=style,
                 attr=attr,
+                panel=panel,
             )
         )
 
@@ -448,35 +633,111 @@ class VuetifyDashboard(VuetifyDocument):
         self.register.scripts.add_resource("plotly")
         return self
 
+    def with_dataframe(
+        self, df, name="dataframe", labels=None, with_rowid=True, rowid_column="rowid"
+    ):
+        if labels is None:
+            labels = list(df.columns)
+        if with_rowid:
+            assert rowid_column not in df.columns
+            columns = [rowid_column] + [c for c in df.columns]
+            df = df.copy()
+            df[rowid_column] = np.arange(len(df))
+            df = df[columns]
+            labels=["#"]+labels
+        self.df=df
+        self.labels=labels
+        r = self.register
+        script = r.vuetify_script
+        script.add_data(name, df.to_json(orient="split"), raw=True)
+        script.add_data("search", "")
+        script.add_data("selected", [])
+        script.add_data(f"{name}_data", [])
+        script.add_created(f"this.{name}_data=this.{name}.data;\n")
+        script.add_data(
+            f"{name}_headers",
+            [
+                dict(text=label, value=str(i), sortable=True)
+                for i, label in enumerate(labels)
+            ],
+        )
+        return self
+
+    def with_row_action(self, code):
+        r = self.register
+        script = r.vuetify_script
+        script.add_method("row_action","function(rowid){%s}"%code)
+        return self
+
+    def with_panel_row_action(self, panel_name):
+        r = self.register
+        script = r.vuetify_script
+        script.add_data("selected_rowid")
+        script.add_data("selected_row", [])
+        return self.with_row_action(f"""
+        this.selected_rowid=rowid;
+        this.selected_row=this.dataframe.data[rowid];
+        this.show_panel('{panel_name}');
+        """)
+
 
 if __name__ == "__main__":
+    import matplotlib.pyplot as plt
     r = Register()
-    doc = VuetifyDashboard(r).with_navigation_drawer().with_app_bar(color="primary").with_plotly()
-    doc.drawer_item("Hello", icon="mdi-home", click="this.alert('hello')")
+    doc = (
+        VuetifyDashboard(r)
+        .with_navigation_drawer()
+        .with_app_bar(color="primary")
+        .with_plotly()
+        .with_panels()
+    )
+    r.home_panel.add("<h1>Home</h1>")
+    doc.panel("panel1", fluid=True).add(
+        "<v-row><v-col><h1>Panel 1</h1>Hello {{what}}!</v-col></v-row>"
+    )
+    doc.panel("panel2").add("<h1>Panel 2</h1>")
+    doc.drawer_item("Home", icon="mdi-home", panel="home_panel")
     doc.drawer_item("Google href", href="http://google.com")
     doc.drawer_item("Google to", to="http://google.com")
     doc.add_bar_button("Hello", click="this.alert('Hello')", color="primary")
     doc.add_bar_menu(
         "Second",
         [
-            dict(
-                title="first",
-                icon="mdi-alert-box",
-                color="green",
-                click="this.alert('Hello1')",
-            ),
-            dict(
-                title="second",
-                icon="mdi-alert-circle",
-                color="secondary",
-                click="this.alert('Hello2')",
-            ),
+            dict(title="first", click="this.alert('Hello1')"),
+            dict(title="second", click="this.alert('Hello2')"),
+            dict(title="Panel 1", panel="panel1"),
+            dict(title="Panel 2", panel="panel2"),
         ],
     )
     doc.add_bar_spacer()
     doc.add_bar_button(None, icon="mdi-magnify", click="this.alert('magnify')")
+    #    doc.with_dataframe(pd.DataFrame(dict(a=[1,2,3],b=[4,5,6])))
+    doc.with_dataframe(pd.read_csv("test.csv")).with_panel_row_action("panel2")
+    #r.vuetify_script.add_data("myfilter",False)
+    r.vuetify_script.add_method("update_filter", """
+    function(){
+        console.log("Update filter",this.myfilter);
+        if (this.myfilter){
+            this.dataframe_data = this.dataframe.data.filter(function(x){
+                return ((x[1]>2000) && (x[1]<2005)); 
+            });
+        }
+        else{
+            this.dataframe_data = this.dataframe.data;
+        }
+    }
+    """)
+    r.vuetify_script.add_watch("myfilter", "function(new_value,old_value){console.log('watch',new_value,old_value);this.update_filter();}")
+    r.panel1.switch("myfilter","My filter", value=False)
+    r.panel1.dataframe_view()
+    r.panel1.add("""{{selected_row}}""")
+    r.panel2.add("""<h2>Selected</h2>{{selected_row}}""")
+    r.panel2.row_detail()
+    plt.plot([0,1],[0,1])
+    r.panel2.figure(plt.gcf())
+    r.panel1.liquer_logo()
 
-    r.app.add("<v-main><v-container>Hello {{what}}!</v-container></v-main>")
+    # r.app.add("<v-main><v-container>Hello {{what}}!</v-container></v-main>")
     #    r.scripts.add(VuetifyScript(r))
     r.vuetify_script.add_data("to_greet", "WORLD")
     r.vuetify_script.add_computed(
